rm(list=ls(all=TRUE))

require(lattice)
require(mnormt)
require(MCMCpack)
require(survival)
require(msm)
require(mi)
require(snowfall)
require(corpcor)
require(quadprog)

########################
###READ IN DATA AND NORMALIZE ACCORDING TO CHEN, ET AL. (2011)
########################
already.imputed=F
if(!already.imputed) {
	
	source("ReadAffyData.R");
	all.data$logtime.imp = all.data$logtime;

  	#Impute censored survival times
	imp.model = predict(survreg(Surv(logtime,event)~AGE+STAGE+GENDER,data=all.data,dist="gaussian"),type="link",se.fit=T)

	for(i in which(all.data$event==0)) {
		all.data$logtime.imp[i] = mean(rtnorm(5000,imp.model$fit[[i]],imp.model$se[[i]],lower=all.data$logtime[i]))
		if(is.na(all.data$logtime.imp[i])) {stop()}
	}

  	#11 pcr measurements (out of 47*91, or about 0.26%) are missing
  	#Impute them with mean values from a multiple imputation
	inf = mi.info(all.data[all.data$A==1,-(1:8)])
	imp.pcr = mi(all.data[all.data$A==1,-(1:8)],info=inf,n.imp=5,n.iter=50,check.coef.convergence=TRUE)
	all.data[all.data$A==1,-(1:8)] = 
    		(mi.completed(imp.pcr)[[1]]+mi.completed(imp.pcr)[[2]]+mi.completed(imp.pcr)[[3]]+mi.completed(imp.pcr)[[4]]+mi.completed(imp.pcr)[[5]])/5

	write.table(all.data,"alldata_clean.txt")
}

####################
#####SET UP NOTATION
####################
all.data = read.table("alldata_clean.txt",header=T)
all.data = all.data[all.data$logtime.imp>0,]
groupA = which(all.data$A==1)
groupB = which(all.data$B==1)
validation.group = which(all.data$training==0)
colnames.pcr = colnames(all.data)[grep("pcr",colnames(all.data))]
colnames.affy = colnames(all.data)[grep("affy",colnames(all.data))]
p = length(colnames.affy)
y = resid(lm(logtime.imp~AGE+STAGE+GENDER,data=all.data,subset=training==1))
fitted.vals.clinical = predict(lm(logtime.imp~AGE+STAGE+GENDER,data=all.data,subset=training==1),newdata=all.data[validation.group,])
y = c(y,all.data[validation.group,"logtime.imp"] -fitted.vals.clinical)

n.b=length(groupB);n.a=length(groupA);

xa = as.matrix(all.data[groupA,colnames.pcr])
location.factor = colMeans(xa);
scale.factor = (colMeans((xa - rep(colMeans(xa),rep(n.a, p)))^2))^.5
s.xa = scale(xa,center=location.factor,scale=scale.factor)
ya = y[groupA]
yb = y[groupB]

new.x = as.matrix(all.data[validation.group,colnames.pcr])		
s.new.x = scale(new.x,center=location.factor,scale=scale.factor)
new.y = as.matrix(y[validation.group])
n.new = length(new.y);

wa = as.matrix(all.data[groupA,colnames.affy])
wb = as.matrix(all.data[groupB,colnames.affy])


####################
#####DO ANALYSIS
####################

	bayes.methods = c("vanilla",
			"hierbetas",
			"ebbetas",
			"ebSigmaX",
			"ebboth")

	methods.with.lambda = c("hierbetas","ebbetas","ebboth");

	bayes.arglist = vector("list",length(bayes.methods))
	names(bayes.arglist) = bayes.methods

	prior.df = 3*p
	bayes.arglist$vanilla = list(hyperparams = list(
						lambda.betas = 0,
						lambda.precision.X = 0,
						precision.X.prior.df = prior.df),
						lambda.betas.update = "none",
						lambda.precision.X.update = "none")
	bayes.arglist$hierbetas = list(hyperparams = list(
						lambda.betas = 0,
						lambda.precision.X = 0,
						precision.X.prior.df = prior.df),
						lambda.betas.update = "bayes",
						lambda.precision.X.update = "none")
	bayes.arglist$ebbetas = list(hyperparams = list(
						lambda.betas = 0,
						lambda.precision.X = 0,
						precision.X.prior.df = prior.df),
						lambda.betas.update = "eb",
						lambda.precision.X.update = "none")
	bayes.arglist$ebSigmaX = list(hyperparams = list(
						lambda.betas = 0,
						lambda.precision.X = 0,
						precision.X.prior.df = prior.df),
						lambda.betas.update = "none",
						lambda.precision.X.update = "eb.invscale")
	bayes.arglist$ebboth = list(hyperparams = list(
						lambda.betas = 0,
						lambda.precision.X = 0,
						precision.X.prior.df = prior.df),
						lambda.betas.update = "eb",
						lambda.precision.X.update = "eb.invscale")	


all.methods = paste(rep(bayes.methods,each=2),rep(c("q","p"),length(bayes.methods)),sep=".")
n.methods = length(all.methods);
results = matrix(0,nrow=p+1,ncol=n.methods);
mspe = matrix(0,nrow=1,ncol=n.methods);
colnames(results) = colnames(mspe) = all.methods;

lambda.matrix = matrix(0,nrow=1,ncol=length(methods.with.lambda))
colnames(lambda.matrix) = methods.with.lambda

predint.coverage = predint.range = matrix(0,nrow=1,ncol=length(bayes.methods));
colnames(predint.coverage) = colnames(predint.range) = bayes.methods;


####################
#####BAYESIAN METHODS
####################
source("Methods.R")
dyn.load(file.path(".",paste("Methods",.Platform$dynlib.ext,sep="")))

mcmc.burnin=4000
mcmc.niter=4000

groups = factor(rep(1:p,each=n.a))

		init.params = list(
			betas = as.numeric(coef(lm(yb~wb)))[-1],
			beta0 = as.numeric(coef(lm(yb~wb)))[1],
			sigma2 = summary(lm(yb~wb))$sigma^2,
			psi = coef(lmer(as.numeric(wa)~as.numeric(xa)+(1|groups)+(as.numeric(xa)|groups)))[[1]][,1],
			nu = coef(lmer(as.numeric(wa)~as.numeric(xa)+(1|groups)+(as.numeric(xa)|groups)))[[1]][,2],
			tau2 = (attributes(VarCorr(lmer(as.numeric(wa)~as.numeric(xa)+(1|groups)+(as.numeric(xa)|groups))))$sc)^2,
			mu.X = xa[3,],
			precision.X = solve(cov.shrink(xa[4:10,])))

rm(groups);

		bayes.arglist$hierbetas$hyperparams$lambda.betas = 
			bayes.arglist$ebbetas$hyperparams$lambda.betas = 	
			bayes.arglist$ebboth$hyperparams$lambda.betas = .5*p*init.params$sigma2/sum(init.params$betas^2)

		bayes.arglist$vanilla$hyperparams$lambda.precision.X = 
			bayes.arglist$hierbetas$hyperparams$lambda.precision.X = 
			bayes.arglist$ebbetas$hyperparams$lambda.precision.X = 	
			bayes.arglist$ebSigmaX$hyperparams$lambda.precision.X =
			bayes.arglist$ebboth$hyperparams$lambda.precision.X = (prior.df-p-1)*diag(diag(var(xa)))


		expanded.new.y = rep(new.y,each=mcmc.niter);
		for(meth in bayes.methods) {
			attach(bayes.arglist[[meth]])
			assign(meth,da.gibbs.C(ya,yb,xa,wa,wb,location.factor,scale.factor,init.params,hyperparams,mcmc.burnin,mcmc.niter,lambda.betas.update,lambda.precision.X.update,single.psi=F,single.nu=F,constrain.lambda.precision.X="diag",s.xa))
			detach(bayes.arglist[[meth]])
			attach(get(meth));
			mspe[,paste(meth,".q",sep="")] = mean((new.y-mean.beta0-new.x%*%mean.betas.qloss)^2)
			mspe[,paste(meth,".p",sep="")] = mean((new.y-mean.beta0-new.x%*%mean.betas.ploss)^2) 
			if(meth %in% methods.with.lambda) {lambda.matrix[,meth] = mean(store.lambda.betas);}	
			foo = apply(store.beta0 + store.betas%*%t(new.x) + sqrt(store.sigma2)*rnorm(mcmc.niter*n.new) - expanded.new.y,2,quantile,p=c(.025,.975));
			predint.range[,meth] = mean(apply(foo,2,diff));
			predint.coverage[,meth] = mean(colMeans(sign(foo))==0);
			results[,paste(meth,".q",sep="")] = c(mean.beta0,mean.betas.qloss)
			results[,paste(meth,".p",sep="")] = c(mean.beta0,mean.betas.ploss)
			detach(get(meth));
		}
		rm(meth);rm(foo);


dyn.unload(file.path(".",paste("Methods",.Platform$dynlib.ext,sep="")))


####################
#####RESULTS
####################

print(mspe)
summary(results)


new.t = all.data$logtime[validation.group]
new.c = 1-all.data$event[validation.group]
aft = survreg(Surv(logtime,event)~AGE+STAGE+GENDER,data=all.data[c(groupA,groupB),],dist="gaussian")
fitted.vals.all = drop(cbind(1,new.x)%*%results)+fitted.vals.clinical



#Brier scores
smod = Surv(new.t,1-new.c);

mat.new.t = matrix(sort(new.t),nrow=length(new.t),ncol=length(new.t),byrow=T)
null.bs = sbrier(smod,matrix(.5,length(new.t),length(new.t)),sort(new.t))[[1]]
sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"vanilla.p"])/sqrt(mean(vanilla$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs
sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"vanilla.q"])/sqrt(mean(vanilla$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs

sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"hierbetas.p"])/sqrt(mean(hierbetas$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs
sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"hierbetas.q"])/sqrt(mean(hierbetas$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs

sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"ebbetas.p"])/sqrt(mean(ebbetas$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs
sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"ebbetas.q"])/sqrt(mean(ebbetas$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs

sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"ebSigmaX.p"])/sqrt(mean(ebSigmaX$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs
sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"ebSigmaX.q"])/sqrt(mean(ebSigmaX$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs

sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"ebboth.p"])/sqrt(mean(ebboth$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs
sbrier(smod,t(pnorm((mat.new.t-fitted.vals.all[,"ebboth.q"])/sqrt(mean(ebboth$store.sigma2)),lower=F)),sort(new.t))[[1]]/null.bs

##compare to AFT using only clinical
sbrier(smod,t(pnorm((mat.new.t-predict(aft,all.data[validation.group,]))/aft$scale,lower=F)),sort(new.t))[[1]]/null.bs



###RISK-INDEXED SURVIVAL PLOTS

#Tertile plots
risk.index= as.numeric(cut(fitted.vals.all[,"ebbetas.p"],c(-Inf,log(30),log(60),Inf),include.lowest=T));
risk.index2= as.numeric(cut(predict(aft,all.data[validation.group,]),c(-Inf,log(30),log(60),Inf),include.lowest=T));

plot.col = gray.colors(3,end=.8)

pdf("SurvivalPlotsTertiles.pdf",height=4)

	par(oma=c(.05,.05,.05,.05))
	par(mar=c(3.1,2.3, .05, 3.1))
	plot.new();plot.window(xlim=c(0,60),ylim=c(0,1))
	lines(survfit(Surv(exp(new.t),1-new.c)~risk.index),xmax=60,conf.int=F,las=1,lwd=2,lty=1,cex.axis=1.2,mark.time=T,col=plot.col)
	lines(survfit(Surv(exp(new.t),1-new.c)~risk.index2),xmax=60,conf.int=F,las=1,lwd=2,lty=2,cex.axis=1.2,mark.time=T,,col=plot.col)
	abline(v=seq(0,60,10), col=rgb(100,100,100,30,max=255))
	abline(h=seq(0,1,.1), col=rgb(100,100,100,30,max=255))
	axis(1);axis(2,las=1);axis(4,las=1);
	legend(title="EBBETAS",c("High","Med","Low"),lwd=2,x=2,y=0.36,bg="white",cex=1,col=plot.col,lty=1);
	legend(title="Clin. Only",c("High","Med","Low"),lwd=2,x=18,y=0.36,bg="white",cex=1,col=plot.col,lty=2);

dev.off()


### gene function
library("AnnotationDbi")
library("GO.db")
library(annotate)
library("hgu133a.db")
library("biomaRt")
gene.ranks = order(abs(results[-1,"ebbetas.p"]/apply(ebbetas$store.betas,2,sd)),decreasing=T)
gene.effect.est = results[-1,"ebbetas.p"]*scale.factor
names(gene.effect.est) = substr(names(gene.effect.est),start=5,stop=100)
gene.names = getSYMBOL(names(gene.effect.est),"hgu133a")
lower.ci = apply(ebbetas$store.betas%*%diag(scale.factor),2,quantile,p=.05)
upper.ci = apply(ebbetas$store.betas%*%diag(scale.factor),2,quantile,p=.95)

gene.annotate = data.frame(gene.names,cbind(gene.effect.est,lower.ci,upper.ci))[gene.ranks,]

subset.to.highlight = 1:10;

pdf("gene_annotate.pdf",width=5);
	par(mar=c(2.1,.05, 2.1, .05))
	plot.new();plot.window(xlim=range(-0.055,0.055),ylim=c(1,91.5+2*length(subset.to.highlight)))
	for(i in 1:91) {
		if(i %in% subset.to.highlight) {
			curr.y.coord = 91+2*length(subset.to.highlight)-3*(i-1)
			curr.pch = "X"
		} else {
			curr.y.coord = 91-i;
			curr.pch = "x";
		}
		points(gene.annotate[i,2],curr.y.coord,pch=curr.pch)
		segments(gene.annotate[i,3],curr.y.coord,gene.annotate[i,4],curr.y.coord,lty=2,col="#AAAAAA")
		if(i %in% subset.to.highlight) {
			effect.sign = sign(gene.annotate[i,2])
			text(x=gene.annotate[i,ifelse(effect.sign==-1,3,4)]+effect.sign*.0007,y=curr.y.coord,gene.annotate[i,1],font=3,adj=c(ifelse(effect.sign==-1,1,0),NA))
		}
	}
	segments(0,1,0,91.5+2*length(subset.to.highlight),lwd=2)
	axis(1,at=seq(-0.04,0.04,by=0.02))	
	axis(3,at=seq(-0.04,0.04,by=0.02))	
dev.off();
